<?xml version="1.0" encoding="UTF-8"?>
<!--
	 *****************************************************************************
run in batch mode only, started from xml spy second file is not written!!!
	*******************************************************************************
	-->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:genxsl="http://www.xml-web.de/genxsl" xmlns:udf="http://www.xml-web.de/udf" exclude-result-prefixes="udf">
	<!-- define output format for stylesheet fragment and namespace alias-->
	<xsl:param name="analyze">new_sheet.xslt</xsl:param>
	<xsl:param name="tracefile">not_processed.xml</xsl:param>
	<xsl:output method="xml" name="nursery" standalone="yes" indent="yes"/>
	<xsl:namespace-alias stylesheet-prefix="genxsl" result-prefix="xsl"/>
	<!-- 
		mind tags already handled in the stylesheet
		read running stylesheet by document()
	 -->
	<xsl:variable name="handled-tags">
		<xsl:for-each select="document($analyze)//xsl:template/@match">
			<xsl:value-of select="."/>
			<xsl:text>, </xsl:text>
		</xsl:for-each>
		<xsl:for-each select="document($analyze)//xsl:value-of/@select">
			<xsl:value-of select="."/>
			<xsl:if test="position() != last()">, </xsl:if>
		</xsl:for-each>
	</xsl:variable>
	<!--
	 *****************************************************************************
	the worker template follows here 
	*******************************************************************************
	-->
	<xsl:template name="tag-trace">
		<!-- ensure xslt 2 processor / at least 1.8 -->
		<xsl:call-template name="check-version">
			<xsl:with-param name="required-version">1.8</xsl:with-param>
		</xsl:call-template>
		<!--
		now write the 'not-yet-processed' stylesheet fragment
		-->
		<xsl:result-document href="{$tracefile}" format="nursery">
			<genxsl:stylesheet version="2.0">
				<xsl:call-template name="write-intro"/>
				<!-- get the tag names of the input file -->
				<xsl:for-each-group select="//*" group-by="name()">
					<xsl:sort select="name()" case-order="lower-first"/>
					<!-- keep name unique -->
					<xsl:variable name="cname" select="name(current-group()[1])"/>
					<xsl:if test="not(contains($handled-tags, $cname))">
						<xsl:call-template name="make-comment">
							<xsl:with-param name="instring" select="$cname"/>
						</xsl:call-template>
						<!-- write template for name found -->
						<genxsl:template match="{$cname}">
							<!-- are there any occurences of attributes for this element??? -->
							<xsl:if test="//*[name() = $cname][@*]">
								<xsl:comment>possible attributes</xsl:comment>
								<xsl:for-each-group select="//*[name() = $cname]/@*" group-by="name()">
									<xsl:sort select="name()"/>
									<!-- tell xslt processor to evaluate now and not write the select expression to output -->
									<genxsl:value-of select="{concat('@',name(current-group()[1]))}"/>
								</xsl:for-each-group>
							</xsl:if>
							<genxsl:apply-templates/>
						</genxsl:template>
					</xsl:if>
				</xsl:for-each-group>
			</genxsl:stylesheet>
		</xsl:result-document>
	</xsl:template>
	<!-- 
some helpers	
-->
	<xsl:template name="write-intro">
		<xsl:comment>*****************************************************************</xsl:comment>
		<xsl:comment>Tags not handeled by templates in current stylesheet </xsl:comment>
		<xsl:comment>
			<xsl:value-of select="udf:impressum(' M. Knobloch ')"/>
		</xsl:comment>
		<xsl:comment>*****************************************************************</xsl:comment>
		<xsl:comment> Handeled tags: 
			<xsl:value-of select="$handled-tags"/>
		</xsl:comment>
	</xsl:template>
	<!-- 
	user defined function only works with xslt 2.0 Processor !!! 
	-->
	<xsl:function name="udf:impressum">
		<xsl:param name="cr"/>
		<xsl:value-of select="concat('&#169; ',$cr, string(current-date()),' using ', system-property('xsl:vendor'), '  V. ', system-property('xsl:version'))"/>
	</xsl:function>
	<!-- 
	check min Version given in global var $required-version
	-->
	<xsl:template name="check-version">
		<xsl:param name="required-version"/>
		<xsl:if test="number(system-property('xsl:version')) &lt; number($required-version)">
			<xsl:message terminate="yes">
				<xsl:text>Sorry, this stylesheet requires XSLT </xsl:text>
				<xsl:value-of select="$required-version"/>
			</xsl:message>
		</xsl:if>
	</xsl:template>
	<xsl:template name="make-comment">
		<xsl:param name="instring">test instring</xsl:param>
		<xsl:variable name="outstring" select="concat('** ', $instring ,' **')"/>
		<xsl:comment>
			<xsl:value-of select="string-pad('*',string-length($outstring) + 10)"/>
		</xsl:comment>
		<xsl:comment>
			<xsl:value-of select="$outstring"/>
		</xsl:comment>
		<xsl:comment>
			<xsl:value-of select="string-pad('*',string-length($outstring) + 10)"/>
		</xsl:comment>
	</xsl:template>
</xsl:stylesheet>
